@@ -17,9 +17,6 @@ typedef struct {
 	int maxname ;
 	char *title ; } graph_s ;
 
-extern sekw* ReadWholeSequence(FILE *in) ;
-
-
 /* this function does the actual drawing of the sequence gc graph */
 void gc_draw_graph(GtkWidget *picture, GdkEventExpose *event, gpointer data) {
 	GdkDrawable *gdkpic ;
@@ -207,7 +204,7 @@ void graph_gc(gpointer dane, guint signal, GtkWidget *kontrolka) {
 	}
 
 	/* read sequence list */
-	for(nseq = 0 ; (tmp = ReadWholeSequence(fp)) != NULL ; nseq++) {
+	for(nseq = 0 ; (tmp = gp_seq_read(fp)) != NULL ; nseq++) {
 
 		if(list == NULL) {
 			list = tmp ; 
