@@ -96,6 +96,22 @@ void graph3d_quit(GtkWidget *w, gpointer dane) {
 
 }
 
+/* used to measure the execution times of the code */
+void checkpoint(struct timeval *t0, char *description) {
+	long usec ;
+	struct timeval t1 ;
+
+	gettimeofday(&t1, NULL) ;
+	usec = 1000000 * (t1.tv_sec - t0->tv_sec) + t1.tv_usec - t0->tv_usec ;
+	printf("%s %li ", description, usec) ;
+	t0->tv_sec = t1.tv_sec ;
+	t0->tv_usec = t1.tv_usec ;
+
+}
+
+
+
+/* -------------------------- handling z-buffer -------------------- */
 
 /* create a new z buffer for storing the pixmap z axis informations */
 buffer3d_s * graph3d_buffer3d_new(int width, int height) {
@@ -121,11 +137,13 @@ void graph3d_buffer3d_free(buffer3d_s * b) {
 }
 
 
+/* -------------------------- drawing primitives --------------------------- */
+
 /* same as gdk_draw_string, but checks the parameters */
 void graph3d_draw_string(GdkDrawable *drawable, GdkFont *font, GdkGC *gc,
 	gint x, gint y, const gchar *string, gint maxw, gint maxh) {
 
-	gint ascent, descent, width, lbearing, rbearing, length ;
+	gint ascent, descent, width, lbearing, rbearing ;
 	
 	gdk_string_extents(font, string, &lbearing, &rbearing, &width,
 		&ascent, &descent) ;
@@ -141,7 +159,7 @@ void graph3d_draw_string(GdkDrawable *drawable, GdkFont *font, GdkGC *gc,
 void graph3d_draw_rectangle(GdkDrawable *pic, buffer3d_s *b, points3d_t pos,
 	int width, int height, GdkGC *pen) {
 
-	int x0, y0, xt, yt, zt, j ;
+	int x0, y0, xt, yt, j ;
 	double z0 ; 
 
 	if(pos[0] < 0 || pos[0] > b->width - width  - 1
@@ -169,7 +187,7 @@ static void graph3d_draw_line(GdkDrawable *pic, buffer3d_s *b, points3d_t start,
 	points3d_t end, GdkGC *pen) {
 
 	int *pp_set0, *pp_set, *pp_z0, *pp_z, 
-		v, i, j, k, xv, yv, x0, y0, xt, yt, x1, y1, z0, zv, z1, zt ;
+		v, i, j, xv, yv, x0, y0, xt, yt, x1, y1, z0, zv, z1, zt ;
 	double vd ;
 
 	if(!b) {
@@ -182,39 +200,6 @@ static void graph3d_draw_line(GdkDrawable *pic, buffer3d_s *b, points3d_t start,
 	x0 = start[0], y0 = start[1], z0 = start[2] ;
 	x1 = end[0], y1 = end[1], z1 = end[2] ;
 
-	/* if(start[0] < 0) x0 = 0 ;
-	else if(start[0] >= b->width) x0 = b->width - 1 ;
-		else x0 = start[0] ;
-
-	if(start[1] < 0) y0 = 0 ;
-	else if(start[1] >= b->height) y0 = b->height - 1 ;
-		else y0 = start[1] ;
-
-	z0 = start[2] ;
-
-	if(end[0] < 0) x1 = 0 ;
-	else if(end[0] >= b->width) x1 = b->width - 1 ;
-		else x1 = end[0] ;
-
-	if(start[1] < 0) y1 = 0 ;
-	else if(end[1] >= b->height) y1 = b->height - 1 ;
-		else y1 = end[1] ;
-
-	z1 = end[2] ;*/
-
-	/* 
-		if(x0 < 0 || x0 >= b->width || 
-		 y0 < 0 || y0 >= b->height ||
-		 x1 < 0 || x1 >= b->width || 
-		 y1 < 0 || y1 >= b->height ) {
-			printf("fatal error in graph3d_draw_line()\n") ;
-			printf("x0 = %i, y0 = %i, x1 = %i, y1 = %i, width %i height %i\n",
-				x0, y0, x1, y1, b->width, b->height) ;
-
-		return ;
-	}
-	*/
-
 	pp_set0 = b->set ;
 	pp_z0 = b->z ;
 	xv = x1 - x0 , yv = y1 - y0 , zv = z1 - z0 ;
@@ -243,12 +228,12 @@ static void graph3d_draw_line(GdkDrawable *pic, buffer3d_s *b, points3d_t start,
 			++(*(pp_set)) ;
 			 *(pp_z) = zt ;
 		}
-
 	}
-
 }
 
 
+/* ------------------------------ vector operations ---------------------- */
+
 /* scales x and y positions of a point according to the current perspective */
 static void graph3d_points_perspective(points3d_t p, double perspective, double maxvect) {
 	double factor ;
@@ -329,19 +314,7 @@ static void graph3d_graph_scale(graph3d_s *graph, double scale) {
 }
 
 
-/* used to measure the execution times of the code */
-void checkpoint(struct timeval *t0, char *description) {
-	long usec ;
-	struct timeval t1 ;
-
-	gettimeofday(&t1, NULL) ;
-	usec = 1000000 * (t1.tv_sec - t0->tv_sec) + t1.tv_usec - t0->tv_usec ;
-	printf("%s %i ", description, usec) ;
-	t0->tv_sec = t1.tv_sec ;
-	t0->tv_usec = t1.tv_usec ;
-
-}
-
+/* ------------------------- drawing the graph ----------------------- */
 
 /* this function does the actual drawing of the sequence gc graph */
 void graph3d_draw_graph(GtkWidget *picture, graph3d_s *graph, opt_s *o) {
@@ -350,12 +323,9 @@ void graph3d_draw_graph(GtkWidget *picture, graph3d_s *graph, opt_s *o) {
 	GdkGC *pen, *black, *white ;
 	GtkStyle *style ;
 	char scale_name[3][2] = { "X\0", "Y\0", "Z\0" } ;
-	double scale, scalex, scaley, perspective ;
+	double perspective ;
 	points3d_t p0, p1 ;
-	int i, j, wid0, wid1  ;
-	int height, width, dheight, dwidth, marginx, marginy ;
-	int leng, color ;
-	data_s *d ;
+	int i, wid0, wid1, dheight, dwidth, color ;
 	buffer3d_s *buf ;
 
 	graph->buffer = graph3d_buffer3d_new(picture->allocation.width, 
@@ -554,11 +524,12 @@ void graph3d_expose(GtkWidget *picture, GdkEventExpose *event, gpointer data) {
 }
 
 
+/* ------------------------ recalculating the whole graph ----------------- */
+
 /* recalculates the coordinates of the vectors */
 void graph3d_turn_degree(graph3d_s * graph, int degree, int axis) {
-	int i, a, b, coord ;
+	int i, a, b ;
 	double ad, bd, teta, sin_t, cos_t  ;
-	long tdif ;
 
 	teta = degree * PI / 180 ;
 
@@ -610,6 +581,7 @@ int graph3d_dircos(points3d_t ax, points3d_t cos) {
 	cos[0] = ax[0] / d ;
 	cos[1] = ax[1] / d ;
 	cos[2] = ax[2] / d ;
+	return EXIT_SUCCESS ;
 }
 
 
@@ -626,7 +598,7 @@ int graph3d_points_cosini_r(points3d_t p,
 	t[2] = cx[2]*p[0] + cy[2]*p[1] + cz[2]*p[2] ;
 
 	graph3d_points_copy(p, t) ;
-
+	return EXIT_SUCCESS ;
 }
 
 
@@ -643,7 +615,7 @@ int graph3d_points_cosini(points3d_t p,
 	t[2] = cz[0]*p[0] + cz[1]*p[1] + cz[2]*p[2] ;
 
 	graph3d_points_copy(p, t) ;
-
+	return EXIT_SUCCESS ;
 }
 
 
@@ -654,7 +626,6 @@ void graph3d_turn_cosini_r (graph3d_s* graph,
 	points3d_t cz) {
 
 	int i ;
-	points3d_t *p ;
 
 	/* recalculating point and projection coordinates */
 	for(i = 0 ; i < graph->npoints ; i++) {
@@ -679,7 +650,6 @@ void graph3d_turn_cosini (graph3d_s* graph,
 	points3d_t cz) {
 
 	int i ;
-	points3d_t *p ;
 
 	/* recalculating point and projection coordinates */
 	for(i = 0 ; i < graph->npoints ; i++) {
@@ -716,6 +686,8 @@ void graph3d_axes_zero(graph3d_s *graph) {
 }
 
 
+/* ------------------------- interface functions ----------------------- */
+
 /* this function is called repeatedly when an arrow is pressed */
 void graph3d_turn(gpointer dane) {
 	GtkWidget *picture ;
@@ -762,7 +734,6 @@ void graph3d_test(GtkWidget *w, gpointer dane) {
 	graph3d_s *graph ;
 	data_s *d ;
 	opt_s *o ;
-	guint *id ;
 	points3d_t cx, cy, cz ;
 
 	d = (data_s *) dane ;
@@ -785,7 +756,6 @@ void graph3d_test(GtkWidget *w, gpointer dane) {
 void graph3d_zoom_increase(GtkWidget *w, gpointer dane) {
 	graph3d_s *graph ;
 	data_s *d ;
-	guint *id ;
 
 	d = (data_s *) dane ;
 	graph = (graph3d_s *) d->data1 ;
@@ -799,7 +769,6 @@ void graph3d_zoom_increase(GtkWidget *w, gpointer dane) {
 void graph3d_zoom_decrease(GtkWidget *w, gpointer dane) {
 	graph3d_s *graph ;
 	data_s *d ;
-	guint *id ;
 
 	d = (data_s *) dane ;
 	graph = (graph3d_s *) d->data1 ;
@@ -810,9 +779,7 @@ void graph3d_zoom_decrease(GtkWidget *w, gpointer dane) {
 
 /* turns the graph once if an arrow is clicked */
 void graph3d_turning_once(GtkWidget *w, gpointer dane) {
-	graph3d_s *graph ;
 	data_s *d ;
-	guint *id ;
 
 	d = (data_s *) dane ;
 	graph3d_turn(d) ;
@@ -820,7 +787,6 @@ void graph3d_turning_once(GtkWidget *w, gpointer dane) {
 
 /* starts turning the graph when an arrow gets pressed */
 void graph3d_turning_start(GtkWidget *w, gpointer dane) {
-	graph3d_s *graph ;
 	data_s *d ;
 	guint *id ;
 
@@ -829,13 +795,11 @@ void graph3d_turning_start(GtkWidget *w, gpointer dane) {
 	*id = gtk_timeout_add(100, (GtkFunction) graph3d_turn, d) ;
 	d->data4 = id ;
 	graph3d_turn(d) ;
-
 }
 
 
 /* stops turning the graph after an arrow is released */
 void graph3d_turning_stop(GtkWidget *w, gpointer dane) {
-	graph3d_s *graph ;
 	data_s *d ;
 	guint *id ;
 
@@ -850,7 +814,6 @@ void graph3d_turning_stop(GtkWidget *w, gpointer dane) {
 void graph3d_toggle_points(GtkWidget *w, gpointer dane) {
 	graph3d_s *graph ;
 	data_s *d ;
-	opt_s *o ;
 
 	d = (data_s *) dane ;
 	graph = (graph3d_s *) d->data1 ;
@@ -863,7 +826,6 @@ void graph3d_toggle_points(GtkWidget *w, gpointer dane) {
 void graph3d_toggle_turning(GtkWidget *w, gpointer dane) {
 	graph3d_s *graph ;
 	data_s *d ;
-	opt_s *o ;
 
 	d = (data_s *) dane ;
 	graph = (graph3d_s *) d->data1 ;
@@ -876,7 +838,6 @@ void graph3d_toggle_turning(GtkWidget *w, gpointer dane) {
 void graph3d_show_coordinates(GtkWidget *w, gpointer dane) {
 	graph3d_s *graph ;
 	data_s *d ;
-	opt_s *o ;
 
 	d = (data_s *) dane ;
 	graph = (graph3d_s *) d->data1 ;
@@ -890,7 +851,6 @@ void graph3d_show_coordinates(GtkWidget *w, gpointer dane) {
 void graph3d_show_perspective(GtkWidget *w, gpointer dane) {
 	graph3d_s *graph ;
 	data_s *d ;
-	opt_s *o ;
 
 	d = (data_s *) dane ;
 	graph = (graph3d_s *) d->data1 ;
@@ -904,7 +864,6 @@ void graph3d_show_perspective(GtkWidget *w, gpointer dane) {
 void graph3d_show_labels(GtkWidget *w, gpointer dane) {
 	graph3d_s *graph ;
 	data_s *d ;
-	opt_s *o ;
 
 	d = (data_s *) dane ;
 	graph = (graph3d_s *) d->data1 ;
@@ -928,123 +887,7 @@ void graph3d_show_projections(GtkWidget *w, gpointer dane) {
 }
 
 
-/* reads graph data from a file, allocates space etc. */
-graph3d_s * graph3d_read_file(opt_s *o, char *filenam) {
-	int alloc = 500, i, j, k, tmp, class ;
-	double x, y, z ;
-	char bufor[BUFSIZ], label[MAXNAME] ;
-	graph3d_s *graph ;
-	FILE *fp ;
-	char *cp ;
-
-	if( (fp = fopen(filenam, "r")) == NULL) {
-		gdk_beep() ;
-		printf("could not open file %s for reading\n", filenam) ;
-		return NULL ;
-	}
-
-	graph = calloc(1, sizeof(*graph)) ;
-	
-	/* reserving memory */
-	graph->coord =  malloc(sizeof(*graph->coord)  * alloc) ;
-	graph->castx =  malloc(sizeof(*graph->coord)  * alloc) ;
-	graph->casty =  malloc(sizeof(*graph->coord)  * alloc) ;
-	graph->castxy = malloc(sizeof(*graph->coord)  * alloc) ;
-	graph->class =  malloc(sizeof(*graph->class)  * alloc) ;
-	graph->labels = malloc(sizeof(*graph->labels) * alloc) ;
-
-	/* defaults */
-	graph->npoints = 0 ;
-	graph->draw_labels = TRUE ;
-	graph->draw_projections = FALSE ;
-	graph->draw_coordinates = TRUE ;
-	graph->turn_around_local = TRUE ;
-	graph->big_points = TRUE ;
-	graph->draw_perspective = FALSE ;
-	graph->zoom = 1.0 ;
-	graph->pixmap = NULL ;
-	graph->buffer = NULL ;
-	graph->title = NULL ;
-
-	if(!graph->coord || !graph->castx || !graph->casty || !graph->castxy) {
-		gdk_beep() ;
-		printf("Could not allocate enough memory\n") ;
-		graph3d_free(graph) ;
-		return NULL ;
-	}
-
-	/* reading the file with the data */
-	for(i = 0, j = 0 ; fgets(bufor, BUFSIZ, fp) ; i++) {
-
-		if(*bufor == '#' || *bufor == '\n') continue ;
-
-		class = 0 ;
-		*label = '\0' ;
-		tmp = sscanf(bufor, " %lf %lf %lf %i %8s", &x, &y, &z, &class, label) ;
-
-		if(tmp < 3) {
-			printf("could not read line %i\n", i) ;
-			continue ;
-		}
-
-		if(tmp == 3) sscanf(bufor, " %lf %lf %lf %8s", &x, &y, &z, label) ;
-
-		/* reserving more memory if necessary */
-		if(j >= alloc) {
-			alloc += 500 ;
-			graph->coord = realloc(graph->coord, sizeof(*graph->coord)*alloc) ;
-			graph->castx = realloc(graph->castx, sizeof(*graph->coord)*alloc) ;
-			graph->casty = realloc(graph->casty, sizeof(*graph->coord)*alloc) ;
-			graph->castxy = realloc(graph->castxy, sizeof(*graph->coord)*alloc) ;
-			graph->labels = realloc(graph->labels, sizeof(*graph->labels) * alloc) ;
-			graph->class =  realloc(graph->class, sizeof(*graph->class)  * alloc) ;
-
-			if(!graph->coord || !graph->castx || !graph->casty || !graph->castxy) {
-				printf("Could not allocate enough memory\n") ;
-				graph3d_free(graph) ;
-				return NULL ;
-			}
-		}
-
-		graph->class[j] = class ;
-		strcpy(graph->labels[j], label) ;
-
-		/* store the coordinates of the XY plane projections */
-		graph->coord[j][0] = x ;
-		graph->coord[j][1] = y ;
-		graph->coord[j][2] = z ;
-
-		graph->castx[j][0] = x ;
-		graph->castx[j][1] = 0 ;
-		graph->castx[j][2] = 0 ;
-
-		graph->casty[j][0] = 0 ;
-		graph->casty[j][1] = y ;
-		graph->casty[j][2] = 0 ;
-
-		graph->castxy[j][0] = x ;
-		graph->castxy[j][1] = y ;
-		graph->castxy[j][2] = 0 ;
-
-		j++ ;
-	}
-
-	if(j < 1) {
-		printf("no data points read\n") ;
-		graph3d_free(graph) ;
-		return NULL ;
-	}
-
-	graph->maxvect = graph3d_maxvect(graph->coord, j) ;
-	graph3d_axes_zero(graph) ;
-	graph->npoints = j ;
-
-	return graph ;
-
-}
-
-
-/* create buttons with tooltpis, shortcut keys and connect the apr. functions */
+/* create buttons with tooltips, shortcut keys and connect the apr. functions */
 void graph3d_create_buttons(
 	GtkWidget *okno, 
 	GtkWidget *ypole, 
@@ -1054,7 +897,7 @@ void graph3d_create_buttons(
 	opt_s *o) {
 	
 	GtkWidget **butt, *habo, *test, *hpole, *labels, 
-		*big, *proj, *coord, *persp, *quit, *zoomp, *zoomm, *toolbar ;
+		*big, *proj, *coord, *persp, *quit, *zoomp, *zoomm ;
 	GtkAccelGroup *accel ;
 	GtkTooltips *tips ;
 	char tmp[300], keys[] = "xXyYzZ", 
@@ -1211,13 +1054,129 @@ void graph3d_create_buttons(
 }
 
 
+/* ------------------------------- data retrieval ------------------------- */
+
+/* reads graph data from a file, allocates space etc. */
+graph3d_s * graph3d_read_file(opt_s *o, char *filenam) {
+	int alloc = 500, i, j, tmp, class ;
+	double x, y, z ;
+	char bufor[BUFSIZ], label[MAXNAME] ;
+	graph3d_s *graph ;
+	FILE *fp ;
+
+	if( (fp = fopen(filenam, "r")) == NULL) {
+		gdk_beep() ;
+		printf("could not open file %s for reading\n", filenam) ;
+		return NULL ;
+	}
+
+	graph = calloc(1, sizeof(*graph)) ;
+	
+	/* reserving memory */
+	graph->coord =  malloc(sizeof(*graph->coord)  * alloc) ;
+	graph->castx =  malloc(sizeof(*graph->coord)  * alloc) ;
+	graph->casty =  malloc(sizeof(*graph->coord)  * alloc) ;
+	graph->castxy = malloc(sizeof(*graph->coord)  * alloc) ;
+	graph->class =  malloc(sizeof(*graph->class)  * alloc) ;
+	graph->labels = malloc(sizeof(*graph->labels) * alloc) ;
+
+	/* defaults */
+	graph->npoints = 0 ;
+	graph->draw_labels = TRUE ;
+	graph->draw_projections = FALSE ;
+	graph->draw_coordinates = TRUE ;
+	graph->turn_around_local = TRUE ;
+	graph->big_points = TRUE ;
+	graph->draw_perspective = FALSE ;
+	graph->zoom = 1.0 ;
+	graph->pixmap = NULL ;
+	graph->buffer = NULL ;
+	graph->title = NULL ;
+
+	if(!graph->coord || !graph->castx || !graph->casty || !graph->castxy) {
+		gdk_beep() ;
+		printf("Could not allocate enough memory\n") ;
+		graph3d_free(graph) ;
+		return NULL ;
+	}
+
+	/* reading the file with the data */
+	for(i = 0, j = 0 ; fgets(bufor, BUFSIZ, fp) ; i++) {
+
+		if(*bufor == '#' || *bufor == '\n') continue ;
+
+		class = 0 ;
+		*label = '\0' ;
+		tmp = sscanf(bufor, " %lf %lf %lf %i %8s", &x, &y, &z, &class, label) ;
+
+		if(tmp < 3) {
+			printf("could not read line %i\n", i) ;
+			continue ;
+		}
+
+		if(tmp == 3) sscanf(bufor, " %lf %lf %lf %8s", &x, &y, &z, label) ;
+
+		/* reserving more memory if necessary */
+		if(j >= alloc) {
+			alloc += 500 ;
+			graph->coord = realloc(graph->coord, sizeof(*graph->coord)*alloc) ;
+			graph->castx = realloc(graph->castx, sizeof(*graph->coord)*alloc) ;
+			graph->casty = realloc(graph->casty, sizeof(*graph->coord)*alloc) ;
+			graph->castxy = realloc(graph->castxy, sizeof(*graph->coord)*alloc) ;
+			graph->labels = realloc(graph->labels, sizeof(*graph->labels) * alloc) ;
+			graph->class =  realloc(graph->class, sizeof(*graph->class)  * alloc) ;
+
+			if(!graph->coord || !graph->castx || !graph->casty || !graph->castxy) {
+				printf("Could not allocate enough memory\n") ;
+				graph3d_free(graph) ;
+				return NULL ;
+			}
+		}
+
+		graph->class[j] = class ;
+		strcpy(graph->labels[j], label) ;
+
+		/* store the coordinates of the XY plane projections */
+		graph->coord[j][0] = x ;
+		graph->coord[j][1] = y ;
+		graph->coord[j][2] = z ;
+
+		graph->castx[j][0] = x ;
+		graph->castx[j][1] = 0 ;
+		graph->castx[j][2] = 0 ;
+
+		graph->casty[j][0] = 0 ;
+		graph->casty[j][1] = y ;
+		graph->casty[j][2] = 0 ;
+
+		graph->castxy[j][0] = x ;
+		graph->castxy[j][1] = y ;
+		graph->castxy[j][2] = 0 ;
+
+		j++ ;
+	}
+
+	if(j < 1) {
+		printf("no data points read\n") ;
+		graph3d_free(graph) ;
+		return NULL ;
+	}
+
+	graph->maxvect = graph3d_maxvect(graph->coord, j) ;
+	graph3d_axes_zero(graph) ;
+	graph->npoints = j ;
+
+	return graph ;
+
+}
+
+
+/* --------------------------- create graph ---------------------------- */
+
 /* creates a graph of the given values */
 int graph3d_create(opt_s *o, char *filenam, char *title) {
 
 	GtkWidget *okno, *ypole, *picture ;
-	GdkDrawable *gdkpic ;
-	GtkStyle *style ;
-	int i, wid, npoints ;
 	graph3d_s *graph ;
 	data_s *d ;
 
@@ -1297,13 +1256,11 @@ void graph3d_create_from_file(gpointer dane, guint signal, GtkWidget *kontrolka)
 /* creates a graph from a text window stored in a okno_s structure */
 void graph3d_create_from_input(gpointer dane, guint signal, GtkWidget *kontrolka) {
 	opt_s *o ;
-	FILE *fp ;
 	char *title ;
 
 	o = (opt_s *) dane ;
 
-	window_tofile(&o->input, o) ;
-	title = strdup(o->input.title) ;
-	graph3d_create(o, o->input.tmpfil, title) ;
-
+	window_tofile(&o->panels[0], o) ;
+	title = strdup(o->panels[0].title) ;
+	graph3d_create(o, o->panels[0].tmpfil, title) ;
 }
