@@ -1,3 +1,4 @@
+/* interface.c - general GUI procedures */
 #include <stdio.h>
 #include <ctype.h>
 #include <stdlib.h>
@@ -7,33 +8,60 @@
 #include "main.h"
 
 
-/* end of the program */
-gint koniec(GtkWidget *kontrolka, gpointer dane) {
-	opt_s *o ;
+void clean_up(opt_s *o) {
+	int i ;
 
-	o = (opt_s *) dane ;
+	/* removing temporary files */
+	for(i = 0 ; i < 3 ; i++) remove(o->panels[i].tmpfil) ;
+
+	/* saving configuration */
+	write_rcfile(o) ;
+
+	/* freeing program entries */
+	free_entries_all(o) ;
+
+	/* freeing colors */
+	free(o->colors) ;
+
+	/* printout of the memory summary */
+	memory_summary() ;
+
+	fprintf(stderr, "Koniec\n") ;
+	exit(EXIT_SUCCESS) ;
+}
+
+
+/* end of the program */
+void arka_end(gpointer dane, guint signal, GtkWidget *kontrolka) {
+	opt_s *o = (opt_s *) dane ;
 
 	/* check if output changed and ask for saving if yes */
 	if(o->panels[1].changed) 
-		if(!demand("Output not saved.\nReally quit?")) return(TRUE) ;
+		if(!demand("Output not saved.\nReally quit?")) return ;
 
-	printf("Koniec\n") ;
 	gtk_main_quit() ;
-	return(FALSE) ;
+
+	clean_up(o) ;
+	return ;
 }
 
 
 /* end of the program: translate functions */
-void zakoncz(gpointer dane, guint signal, GtkWidget *kontrolka) {
-	koniec(kontrolka, dane) ;
+void main_window_destroy(GtkWidget *kontrolka, gpointer dane) {
+	opt_s *o = (opt_s *) dane ;
+
+	if(o->panels[1].changed && demand("Output not saved.\nSave?")) {
+		file_save_lowlevel(o, 1, o->panels[1].filenam) ;
+	}
+
+	clean_up(dane) ;
 }
 
 
 /* end of a gtk_main_quit dialog */
 void zakoncz_dialog(GtkWidget *kontrolka, gpointer dane) {
-	GtkWidget *okno ;
+	GtkWidget *okno = (GtkWidget *) dane ;
 
-	okno = (GtkWidget *) dane ;
 	gtk_widget_destroy(okno) ;
 	gtk_main_quit() ;
 }
@@ -46,7 +74,7 @@ void zakoncz_dialog(GtkWidget *kontrolka, gpointer dane) {
 GtkWidget *create_separator(GtkWidget *parentbox) {
 	GtkWidget *res ;
 	res = gtk_hseparator_new() ;
-	gtk_box_pack_start(GTK_BOX(parentbox), res, FALSE, FALSE, 5) ;
+	gtk_box_pack_start(GTK_BOX(parentbox), res, FALSE, FALSE, 0) ;
 	gtk_widget_show(res) ;
 	return res ;
 }
@@ -56,7 +84,7 @@ GtkWidget *create_separator(GtkWidget *parentbox) {
 GtkWidget *create_label(GtkWidget *parentbox, char* label) {
 	GtkWidget *res ;
 	res = gtk_label_new(label) ;
-	gtk_box_pack_start(GTK_BOX(parentbox), res, FALSE, FALSE, 5) ;
+	gtk_box_pack_start(GTK_BOX(parentbox), res, FALSE, FALSE, 0) ;
 	gtk_widget_show(res) ;
 	return res ;
 }
@@ -66,7 +94,7 @@ GtkWidget *create_label(GtkWidget *parentbox, char* label) {
 GtkWidget *create_hbox(GtkWidget *parentbox) {
 	GtkWidget *hpole ;
 	hpole = gtk_hbox_new(FALSE, 5) ;
-	gtk_box_pack_start(GTK_BOX(parentbox), hpole, FALSE, FALSE, 5) ;
+	gtk_box_pack_start(GTK_BOX(parentbox), hpole, FALSE, FALSE, 0) ;
 	gtk_widget_show(hpole) ;
 	return hpole ;
 }
@@ -79,7 +107,7 @@ GtkWidget *create_combo(GtkWidget* parentbox, char *label, char *items[], int n)
 	int i ;
 
 	hpole = gtk_hbox_new(FALSE, 5) ;
-	gtk_box_pack_start(GTK_BOX(parentbox), hpole, FALSE, FALSE, 5) ;
+	gtk_box_pack_start(GTK_BOX(parentbox), hpole, FALSE, FALSE, 0) ;
 	res = gtk_combo_new() ;
 	gtk_box_pack_start(GTK_BOX(hpole), res, TRUE, TRUE, 5) ;
 
@@ -107,7 +135,7 @@ GtkWidget *create_list(GtkWidget* parentbox, char *label, char *items[], int n)
 	int i ;
 
 	hpole = gtk_hbox_new(FALSE, 5) ;
-	gtk_box_pack_start(GTK_BOX(parentbox), hpole, TRUE, TRUE, 5) ;
+	gtk_box_pack_start(GTK_BOX(parentbox), hpole, TRUE, TRUE, 0) ;
 
 	/* creating item list */
 	for(i = 0 ; i < n ; i++) {
@@ -167,7 +195,7 @@ GtkWidget* create_text(GtkWidget *parentbox, char* text, int editable) {
 		GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC) ;
 	gtk_container_add(GTK_CONTAINER(scrol), res) ;
 
-	gtk_box_pack_start(GTK_BOX(parentbox), scrol, TRUE, TRUE, 5) ;
+	gtk_box_pack_start(GTK_BOX(parentbox), scrol, TRUE, TRUE, 0) ;
 	gtk_widget_show_all(parentbox) ;
 
 	return res ;
@@ -244,6 +272,28 @@ void clear_window(gpointer dane, guint signal, GtkWidget *kontrolka) {
 }
 
 
+/* launches an external editor */
+void external_editor(gpointer dane, guint signal, GtkWidget *kontrolka) {
+	opt_s *o ;
+	okno_s *okno ;
+	char *cmd ;
+
+	o = (opt_s *) dane ;
+
+	okno = &o->panels[signal] ;
+
+	window_tofile(okno, o) ; /* save window contents */
+	cmd = my_strsubst(o->external_editor, "%input", okno->tmpfil) ;
+	/* execute external editor */
+
+	if(system(cmd))
+		komunikat("There was a problem running command:\n%s", o->external_editor); 
+
+	clear_window(dane, signal, kontrolka) ; /* clear window */
+	file_to_window(o, okno, 0) ; /* load edited file again into the window */
+}
+
+
 /* switches window showing on and off */
 void toggle_windows(gpointer dane, guint signal, GtkWidget *kontrolka) {
 	opt_s *o ;
@@ -265,6 +315,29 @@ void toggle_windows(gpointer dane, guint signal, GtkWidget *kontrolka) {
 }
 
 
+/* plops the contents from one window to the other one */
+void plop_contents(gpointer dane, guint signal, GtkWidget *kontrolka) {
+	opt_s *o ;
+	okno_s *okno1, *okno2 ;
+	char *tmp ;
+	int i = 0 ;
+
+	o = (opt_s *) dane ;
+
+	okno1 = &o->panels[signal >> 8] ;
+	okno2 = &o->panels[signal - 256 * (signal >> 8)] ;
+
+	say(0, "%s => %s", okno1->title, okno2->title) ;
+
+	tmp = gtk_editable_get_chars(GTK_EDITABLE(okno1->tekst), 0, -1) ;
+	gtk_editable_delete_text(GTK_EDITABLE(okno2->tekst), 0, -1) ;
+	gtk_editable_insert_text(GTK_EDITABLE(okno2->tekst), tmp, strlen(tmp), &i) ;
+	free(tmp) ;
+	
+}
+
+
+
 /* toggles the input window editable / not editable */
 void toggle_editable(gpointer dane, guint signal, GtkWidget *kontrolka) {
 	opt_s *o ;
@@ -351,7 +424,7 @@ char *get_font_name(opt_s *o, char *name) {
 	GtkFontSelectionDialog *wybcz_d ;
 	data_s d ;
 	char *res ;
-	Warn("get_font_name") ;
+	say(0, "get_font_name") ;
 
 	wybcz = gtk_font_selection_dialog_new("Choose a font for the main window") ;
 	wybcz_d = GTK_FONT_SELECTION_DIALOG(wybcz) ;
@@ -454,7 +527,6 @@ int append_entries(opt_s *o) {
 
 /* here, the main menu and main window is created */
 void inicjuj_menu(opt_s *o) {
-	
 	int nel ;
 
 	static GtkItemFactoryEntry elementy[] = {
@@ -462,21 +534,32 @@ void inicjuj_menu(opt_s *o) {
 		{"/File/tearoff1",	NULL,	NULL, 0, "<Tearoff>"},
 		/* {"/File/_New",	"<control>N",	NULL,	0}, */
 		{"/File/_Open input file", "<control>O",	file_open,	0},
-		{"/File/_Save output", "<control>S",	file_save,	1},
-		{"/File/S_ave output as", "<control>A",	file_save_as,	1},
+		{"/File/Quick _save output", "<control>S",	file_save,	1},
+		{"/File/S_ave.../Save _input as", NULL,	file_save_as,	0},
+		{"/File/S_ave.../Save _output as", NULL,	file_save_as,	1},
+		{"/File/S_ave.../Save _error as", NULL,	file_save_as,	2},
 		{"/File/sep1",	NULL,	NULL,	0, "<Separator>"},
-		{"/File/_Quit",	"<control>Q",	zakoncz,	0},
+		{"/File/_Quit",	"<control>Q",	arka_end,	0},
 
 		{"/_Edit",	NULL, 0, 0, "<Branch>"},
 		{"/Edit/tearoff1",	NULL,	NULL, 0, "<Tearoff>"},
-		{"/Edit/Clear _output",	"<control>C",	clear_window,	1},
+		{"/Edit/_Search", NULL, search, 0},
+		{"/Edit/Search & _replace", NULL, search, 1},
+		{"/Edit/_Clear.../Clear _input",	NULL,	clear_window,	0},
+		{"/Edit/_Clear.../Clear _output",	"<control>C",	clear_window,	1},
+		{"/Edit/_Clear.../Clear _error",	NULL,	clear_window,	2},
+		{"/Edit/_External editor.../Edit _input", NULL,	external_editor,	0},
+		{"/Edit/_External editor.../Edit _output", NULL,	external_editor,	1},
+		{"/Edit/_External editor.../Edit _error", NULL,	external_editor,	2},
+		{"/Edit/_Output => input", NULL,	plop_contents,	(1<<8)},
+		{"/Edit/_Sequence editor", NULL,	seqed_start,	0},
 
 		{"/_Options", NULL,	0,	0, "<Branch>"},
 		{"/Options/tearoff1",	NULL,	0, 0, "<Tearoff>"},
 		{"/Options/_Editable",	NULL,	toggle_editable,	0, "<CheckItem>"},
 		{"/Options/_Hide tooltips",	NULL,	toggle_tooltips,	0, "<CheckItem>"},
 		{"/Options/_Font",	NULL,	change_font,	0},
-		{"/Options/_Preferences",	NULL,	options_preferences_set,	0},
+		{"/Options/_Preferences...",	NULL,	options_preferences_set,	0},
 
 		{"/_Tools", NULL,	0,	0, "<Branch>"},
 		{"/Tools/tearoff1",	NULL,	NULL, 0, "<Tearoff>"},
@@ -485,6 +568,7 @@ void inicjuj_menu(opt_s *o) {
 		{"/Tools/Graph 3D/_From input", NULL, graph3d_create_from_input, 0},
 		{"/Tools/Graph 3D/_From file", NULL, graph3d_create_from_file, 0},
 		{"/Tools/_Command...",	NULL,	command,	0},
+		{"/Tools/_Sequence editor", NULL,	seqed_start,	0},
 
 		{"/_Programs", NULL,	0,	0, "<Branch>"},
 		{"/Programs/tearoff1",	NULL,	NULL, 0, "<Tearoff>"},
@@ -541,18 +625,22 @@ void insert_text(GtkEditable *w, const gchar *text, gint l, gint *p, gpointer da
 
 /* initiates the color map */
 int allocate_colors(opt_s *o) {
-	int i, n = 8 ;
+	int i, n ;
 
+	n = WHITE + 1 ;
+
+	say(0, "Allocating colors") ;
 	o->ncolors = n ;
 	o->colors = malloc(n * sizeof(*o->colors)) ;
-	gdk_color_parse("black", &o->colors[0]) ;
-	gdk_color_parse("red", &o->colors[1]) ; 
-	gdk_color_parse("green", &o->colors[2]) ;
-	gdk_color_parse("blue", &o->colors[3]) ;
-	gdk_color_parse("violet", &o->colors[4]) ;
-	gdk_color_parse("yellow", &o->colors[5]) ;
-	gdk_color_parse("magenta", &o->colors[6]) ;
-	gdk_color_parse("cyan", &o->colors[7]) ; 
+	gdk_color_parse("black", &o->colors[BLACK]) ;
+	gdk_color_parse("red", &o->colors[RED]) ; 
+	gdk_color_parse("green", &o->colors[GREEN]) ;
+	gdk_color_parse("blue", &o->colors[BLUE]) ;
+	gdk_color_parse("violet", &o->colors[VIOLET]) ;
+	gdk_color_parse("yellow", &o->colors[YELLOW]) ;
+	gdk_color_parse("magenta", &o->colors[MAGENTA]) ;
+	gdk_color_parse("cyan", &o->colors[CYAN]) ; 
+	gdk_color_parse("white", &o->colors[WHITE]) ; 
 
 	for(i = 0; i < n; i++) {
 		if(!gdk_colormap_alloc_color(gdk_colormap_get_system(), &o->colors[i], FALSE, TRUE)) 
@@ -594,16 +682,18 @@ void make_ramka(GtkWidget *kontrolka, okno_s *o) {
 /* initiates the main window and calls the inicjuj_menu function */
 void inicjuj_main_window(opt_s *o) {
 	int i ;
-	GtkWidget *panel1, *panel2 ;
+	char tmp[100] ;
+	GtkWidget *w, *panel1, *panel2 ;
 
 	/* creating main window */
 	o->okno = gtk_window_new(GTK_WINDOW_TOPLEVEL) ;
 	gtk_widget_show(o->okno) ;
 	gtk_widget_set_usize(o->okno,
 		gdk_screen_width() * 0.8, gdk_screen_height() * 0.8) ;
-	gtk_window_set_title(GTK_WINDOW(o->okno),"Arka") ;
+	sprintf(tmp, "Arka v. %s", VERSION) ;
+	gtk_window_set_title(GTK_WINDOW(o->okno), tmp) ;
 	gtk_signal_connect(GTK_OBJECT(o->okno), "destroy", 
-		GTK_SIGNAL_FUNC(koniec), o) ;
+		GTK_SIGNAL_FUNC(main_window_destroy), o) ;
 
 	/* vertical container */
 	o->ypole = gtk_vbox_new(FALSE, 1) ;
@@ -649,6 +739,12 @@ void inicjuj_main_window(opt_s *o) {
 		"You will find here any error messages issued by the programs run."
 		"In other words, it corresponds to the stderr", "Just try it") ;
 
+	w = gtk_item_factory_get_widget(o->fabryka, "/Options/Hide tooltips") ;
+	i = o->show_tooltips ;
+	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(w), !o->show_tooltips) ;
+	o->show_tooltips = i ;
+	if(o->show_tooltips) gtk_tooltips_enable(o->tips) ;
+	else gtk_tooltips_disable(o->tips) ;
 
 	/* status bar */
 	o->status = gtk_statusbar_new () ;
