@@ -1045,7 +1045,7 @@ void programs_reload_all(gpointer dane, guint signal, GtkWidget *kontrolka) {
 /* given an entry, append it to the main menu */
 int append_item_to_menu(opt_s *o, entry_s *e) {
 	entry_s *tmp ;
-	char *t ;
+	char *t, *t2 ;
 	int i ;
 
 	/* preparing item_factory item */
@@ -1085,9 +1085,12 @@ int parse_and_append(opt_s *o, FILE* fp, int start) {
 	i = start ; 
 
 	while( (e = parse_rc(fp, &line)) != NULL ) {
+		say(0, "parse_and_append: appending %s", e->entry) ;
 		e->number = i ;
 		append_item_to_menu(o, e) ;
 		i++ ;
+		say(0, "parse_and_append: entry %i appended", e->number) ;
+		
 	}
 
 	rebuild_menu(o) ;
@@ -1566,6 +1569,7 @@ entry_s* parse_program(char *line) {
 	q = strchr(line, ')') ;
 	if(q) { *q = '\0' ; q++ ; }
 	res->name = my_strdup(p) ;
+	say(0, "parse_program: program %s", res->name) ;
 
 	p = strchr(q, '(') ;
 	if(!p) {
@@ -1737,6 +1741,8 @@ entry_s * parse_rc(FILE *in, int *line) {
 
 	result = NULL ;
 
+	say(0, "parse_rc: warming up") ;
+
 	/* skipping to the first start(program) entry */
 	while( !program_found && (fgets(bufor, BUFSIZ, in)) != NULL) {
 		(*line)++ ;
@@ -1761,6 +1767,7 @@ entry_s * parse_rc(FILE *in, int *line) {
 
 	while(!program_end_found && (fgets(bufor, BUFSIZ, in)) != NULL) {
 		(*line)++ ;
+		say(0, "parse_rc: reading line %i", *line) ;
 
 		/* skipping comments and empty lines */
 		if(*bufor == '\n' || *bufor == '#') continue ;
